#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

extern Xdialog_data Xdialog;

/* This function is called when a "delete_event" is received from the window
 * manager. It is used to trigger a "destroy" event by returning FALSE
 * (provided the "--no-close" option was not given).
 */
gboolean delete_event(gpointer object, gpointer data)
{
	return Xdialog.no_close;
}

/* This function is called when a "destroy" event is received 
 * by the top level window.
 */
gboolean destroy_event(gpointer object, gpointer data)
{
	if (Xdialog.timer != 0) {
		gtk_timeout_remove(Xdialog.timer);
		Xdialog.timer = 0;
	}
	gtk_main_quit();
	Xdialog.window = Xdialog.widget1 = Xdialog.widget2 = Xdialog.widget3 = NULL;

	if (Xdialog.file != NULL) {
		if (Xdialog.file != stdin)
			fclose(Xdialog.file);
		Xdialog.file = NULL;
	}
	if (Xdialog.array != NULL) {
		g_free(Xdialog.array);
		Xdialog.array = NULL;
	}
	if (Xdialog.print)
		unlink(TEMP_FILE);

	return FALSE;
}

/* These are the normal termination callback routines that are used by the
 * OK, Cancel and help buttons. They set the exit_code global variable (for
 * use as the exit() parameter by main.c), and then call the destroy_event
 * function that will cleanup everything, destroy the top level window and
 * set the gtk main loop exit flag. They are used "as is" by most Xdialog
 * widgets but care must be taken that they are connected with the
 * gtk_signal_connect_after() function so that any other signal callbacks
 * are executed BEFORE them (after them, the widget does not exists anymore).
 */
gboolean exit_ok(gpointer object, gpointer data)
{
	gtk_widget_destroy(Xdialog.window);
	Xdialog.exit_code = 0;
	return FALSE;
}

gboolean exit_cancel(gpointer object, gpointer data)
{
	gtk_widget_destroy(Xdialog.window);
	Xdialog.exit_code = 1;
	return FALSE;
}

gboolean exit_help(gpointer object, gpointer data)
{
	gtk_widget_destroy(Xdialog.window);
	Xdialog.exit_code = 2;
	return FALSE;
}

gboolean exit_previous(gpointer object, gpointer data)
{
	gtk_widget_destroy(Xdialog.window);
	Xdialog.exit_code = 3;
	return FALSE;
}

/* infobox callbacks: the infobox_timeout_exit() is responsible for
 * closing the infobox once the timeout is over (it therefore calls
 * exit_ok()). The infobox_timeout() function is responsible for
 * reading the stdin and changing the infobox label and/or exiting the
 * infobox by calling exit_ok().
 */

gboolean infobox_timeout_exit(gpointer data)
{
	return exit_ok(NULL, NULL);
}

gboolean infobox_timeout(gpointer data)
{
	char temp[256];
	gchar new_label[MAX_LABEL_LENGTH];
	int ret;

	/* Read the the new label from stdin */
	ret = scanf("%255s", temp);
	if (ret == EOF)
		return exit_ok(NULL, NULL);
	if (ret != 1)
		return TRUE;
	/* If this is a new label marker, then get the new label and
	 * set it up... */
	if (strcmp(temp, "XXX") == 0) {
		new_label[0] = 0;
		while ((ret = scanf ("%255s", temp)) == 1) {
			if (strcmp(temp, "XXX") == 0)
				break;
			if (strlen(new_label)+strlen(temp)+2 < MAX_LABEL_LENGTH) {
				if (strcmp(temp, "\\n") == 0)
					strcat(new_label, "\n");
				else {
					strcat(new_label, " ");
					strcat(new_label, temp);
				}
			}
		}
		if (ret != 1)
			return exit_ok(NULL, NULL);
		if (strlen(new_label) != 0)
			gtk_label_set_text(GTK_LABEL(Xdialog.widget1),
					   new_label);
	} else if (strcmp(temp, "XXXX") == 0)
		return exit_ok(NULL, NULL);

	/* As this is a timeout function, return TRUE so that it
	 * continues to get called */
	return TRUE;
}

/* gauge timeout callback */

gboolean gauge_timeout(gpointer data)
{
	gfloat new_val;
	GtkAdjustment *adj;
	char temp[256];
	gchar new_label[MAX_LABEL_LENGTH];
	int ret;

	/* Read the new progress bar value or the new label from stdin */
	ret = scanf("%255s", temp);
	if (ret == EOF)
		return exit_ok(NULL, NULL);
	if (ret != 1)
		return TRUE;
	/* If this is a new label marker, then get the new label and
	 * set it up... */
	if (strcmp(temp, "XXX") == 0) {
		new_label[0] = 0;
		while ((ret = scanf ("%255s", temp)) == 1) {
			if (strcmp(temp, "XXX") == 0)
				break;
			if (strlen(new_label)+strlen(temp)+2 < MAX_LABEL_LENGTH) {
				if (strcmp(temp, "\\n") == 0)
					strcat(new_label, "\n");
				else {
					strcat(new_label, " ");
					strcat(new_label, temp);
				}
			}
		}
		if (ret != 1)
			return exit_ok(NULL, NULL);
		if (strlen(new_label) != 0)
			gtk_label_set_text(GTK_LABEL(Xdialog.widget2),
					   new_label);
		return TRUE;
	}

	/* Else, try to convert the string into an integer for use as the new
	 * progress bar value... */
	new_val = atoi(temp);
	adj = GTK_PROGRESS(Xdialog.widget1)->adjustment;
	if ((new_val > adj->upper) || (new_val < adj->lower))
		return exit_ok(NULL, NULL);

	/* Set the new value */
	gtk_progress_set_value(GTK_PROGRESS(Xdialog.widget1), new_val);

	/* As this is a timeout function, return TRUE so that it
	 * continues to get called */
	return TRUE;
}

/* tailbox callbacks */

gboolean tailbox_timeout(gpointer data)
{
	gchar buffer[256];
	int nchars;

	do {
		nchars = fread(buffer, sizeof(gchar), 256, Xdialog.file);
		if (nchars == 0)
			break;
		gtk_text_insert(GTK_TEXT(Xdialog.widget1), NULL, NULL, NULL,
				buffer, nchars);
	} while (nchars == 256);

	return TRUE;
}

gint tailbox_keypress(GtkWidget *widget, GdkEventKey *event,
		      gpointer data)
{
	if (event->type == GDK_KEY_PRESS && (event->keyval == GDK_Return ||
					     event->keyval == GDK_KP_Enter)) {
		Xdialog.exit_code = 0;
		return(FALSE);
	}
	return(TRUE);
}

/* Here are the callback functions for the inputbox (OK button callback and
 * entry RETURN/ENTER keypress callback).
 */
gint inputbox_ok(gpointer object, gpointer data)
{
	fprintf(Xdialog.output, "%s",
		gtk_entry_get_text(GTK_ENTRY(Xdialog.widget1)));
	if (Xdialog.widget2 != NULL)
		fprintf(Xdialog.output, "%s%s", Xdialog.separator,
			gtk_entry_get_text(GTK_ENTRY(Xdialog.widget2)));
	if (Xdialog.widget3 != NULL)
		fprintf(Xdialog.output, "%s%s", Xdialog.separator,
			gtk_entry_get_text(GTK_ENTRY(Xdialog.widget3)));
	fprintf(Xdialog.output, "\n");
		
	return TRUE;
}

gboolean inputbox_timeout(gpointer data)
{
	return inputbox_ok(NULL, NULL);
}


gint input_keypress(GtkWidget *entry, GdkEventKey *event, gpointer data)
{
	if (event->type == GDK_KEY_PRESS && (event->keyval == GDK_Return ||
					     event->keyval == GDK_KP_Enter)) {
		if (Xdialog.default_no) {
			Xdialog.exit_code = 1;
		} else {
			inputbox_ok(NULL, NULL);
			Xdialog.exit_code = 0;
		}
		gtk_widget_destroy(Xdialog.window);
		return FALSE;
	}
	return TRUE;
}

/* editbox callback */

gboolean editbox_ok(gpointer object, gpointer data)
{
	int length, i;

	length = gtk_text_get_length(GTK_TEXT(Xdialog.widget1));
	for (i = 0; i < length; i++)
		fputc(GTK_TEXT_INDEX(GTK_TEXT(Xdialog.widget1), i),
		      Xdialog.output);

	return TRUE;
}

/* The print button callback (used by editbox, textbox and tailbox) */

gboolean print_text(gpointer object, gpointer data)
{
	int length, i;
	gchar * buffer;
	char cmd[MAX_PRTCMD_LENGTH];
	FILE * temp;

	strcpysafe(cmd, PRINTER_CMD, MAX_PRTCMD_LENGTH);
	if (strlen(Xdialog.printer) != 0) {
		strcatsafe(cmd, " "PRINTER_CMD_OPTION, MAX_PRTCMD_LENGTH);
		strcatsafe(cmd, Xdialog.printer, MAX_PRTCMD_LENGTH);
	}
	strcatsafe(cmd, " "TEMP_FILE, MAX_PRTCMD_LENGTH);

	length = gtk_text_get_length(GTK_TEXT(Xdialog.widget1));
	buffer = g_malloc((length+1)*sizeof(gchar));
	for (i = 0; i < length; i++)
		buffer[i] = GTK_TEXT_INDEX(GTK_TEXT(Xdialog.widget1), i);

	temp = fopen(TEMP_FILE, "w");
	if (temp != NULL) {
		i = fwrite(buffer, sizeof(gchar), length, temp);
		fclose(temp);
		if (i == length)
			system(cmd);
	}
	g_free(buffer);

	return TRUE;
}

/* rangebox callbacks */

gboolean rangebox_exit(GtkButton *button, gpointer data)
{
	GtkAdjustment *adj;

	adj = GTK_ADJUSTMENT((GtkObject *) Xdialog.widget1);
	fprintf(Xdialog.output, "%d", (gint) adj->value);
	if (Xdialog.widget2 != NULL) {
		adj = GTK_ADJUSTMENT((GtkObject *) Xdialog.widget2);
		fprintf(Xdialog.output, "%s%d", Xdialog.separator,
			(gint) adj->value);
	}
	if (Xdialog.widget3 != NULL) {
		adj = GTK_ADJUSTMENT((GtkObject *) Xdialog.widget3);
		fprintf(Xdialog.output, "%s%d", Xdialog.separator,
			(gint) adj->value);
	}
	fprintf(Xdialog.output, "\n");

	return TRUE;
}


gboolean rangebox_timeout(gpointer data)
{
	return rangebox_exit(NULL, NULL);
}

/* Double-click event is processed as a button click in menubox, radiolist and
 * checklist... The button widget is to be passed as "data".
 */
gint double_click_event(GtkWidget *widget, GdkEventButton *event,
			gpointer data)
{
	if (event->type == GDK_2BUTTON_PRESS || event->type == GDK_3BUTTON_PRESS)
		gtk_signal_emit_by_name(GTK_OBJECT(data), "clicked");

	return FALSE;
}

/* radiolist and checklist callbacks */

void item_toggle(GtkWidget *widget, int i)
{
	if (GTK_TOGGLE_BUTTON(widget)->active) {
		Xdialog.array[i].state = 1;
	} else
		Xdialog.array[i].state = 0;
}

gboolean print_items(GtkButton *button, gpointer data)
{
	int i;
	gboolean flag = FALSE;

	for (i = 0 ; Xdialog.array[i].state != -1 ; i++) {
		if (Xdialog.array[i].state) {
			if (flag)
				fprintf(Xdialog.output, "%s", Xdialog.separator);
			fprintf(Xdialog.output, "%s", Xdialog.array[i].tag);
			flag = TRUE;
		}
	}
	if (flag)
		fprintf(Xdialog.output, "\n");

	return TRUE;
}

gboolean itemlist_timeout(gpointer data)
{
	return print_items(NULL, NULL);
}

/* menubox callback */

void item_select(GtkWidget *widget, gint row, gint column,
		 GdkEventButton *event, gpointer data)
{
	/* Just remember which row was last selected (we use the first
	 * row status variable IOT do so)...
	 */
	Xdialog.array[0].state = row;
}

/* menubox and treeview callback */

gboolean print_selection(GtkButton *button, gpointer data)
{
	fprintf(Xdialog.output, "%s\n", Xdialog.array[Xdialog.array[0].state].tag);

	return TRUE;
}

gboolean menu_timeout(gpointer data)
{
	return print_selection(NULL, NULL);
}

/* treeview callback */

void cb_selection_changed(GtkWidget *tree)
{
	GList *list;
	GtkWidget *item;
	int i;

	list = GTK_TREE_SELECTION(tree);
	if (list != NULL) {
		item = GTK_WIDGET(list->data);
		for (i = 0 ; Xdialog.array[i].state != -1 ; i++) {
			if (Xdialog.array[i].widget == item) {
				Xdialog.array[0].state = i;
				break;
			}
		}
	}
}

/* buildlist callbacks */

gboolean add_to_list(GtkButton *button, gpointer data)
{
	GList *selected;

	selected = g_list_copy(GTK_LIST(Xdialog.widget1)->selection);
	gtk_list_remove_items_no_unref(GTK_LIST(Xdialog.widget1), selected);
	gtk_list_append_items(GTK_LIST(Xdialog.widget2), selected);

	return TRUE;
}

gboolean remove_from_list(GtkButton *button, gpointer data)
{
	GList *selected;

	selected = g_list_copy(GTK_LIST(Xdialog.widget2)->selection);
	gtk_list_remove_items_no_unref(GTK_LIST(Xdialog.widget2), selected);
	gtk_list_append_items(GTK_LIST(Xdialog.widget1), selected);

	return TRUE;
}

gboolean print_list(GtkButton *button, gpointer data)
{
	GList *children;
	int i;
	gboolean flag = FALSE;

	children = GTK_LIST(Xdialog.widget2)->children;
	while (children != NULL) {
		for (i = 0 ; Xdialog.array[i].state != -1 ; i++) {
			if (Xdialog.array[i].widget == children->data) {
				if (flag)
					fprintf(Xdialog.output, "%s", Xdialog.separator);
				fprintf(Xdialog.output, "%s", Xdialog.array[i].tag);
				flag = TRUE;
				break;
			}
		}
		children = g_list_next(children);
	}
	if (flag)
		fprintf(Xdialog.output, "\n");

	return TRUE;
}

gboolean buildlist_timeout(gpointer data)
{
	return print_list(NULL, NULL);
}

/* fselect and dselect callback */

gboolean filesel_exit(GtkWidget *filesel, gpointer client_data)
{
	fprintf(Xdialog.output, "%s\n",
		gtk_file_selection_get_filename(GTK_FILE_SELECTION(client_data)));
	return exit_ok(NULL, NULL);
}

/* calendar callbacks */

gboolean calendar_exit(gpointer object, gpointer data)
{
	gint day, month, year;

	gtk_calendar_get_date(GTK_CALENDAR(Xdialog.widget1), &year, &month, &day);
	fprintf(Xdialog.output, "%02d/%02d/%d\n", day, month+1, year);

	return TRUE;
}

gboolean calendar_timeout(gpointer data)
{
	return calendar_exit(NULL, NULL);
}

/* timebox callbacks */

gboolean timebox_exit(gpointer object, gpointer data)
{
	fprintf(Xdialog.output, "%02d:%02d:%02d\n",
		gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(Xdialog.widget1)),
		gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(Xdialog.widget2)),
		gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(Xdialog.widget3)));

	return TRUE;
}

gboolean timebox_timeout(gpointer data)
{
	return timebox_exit(NULL, NULL);
}
