/* interface.c for Xdialog.*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "yes.xpm"
#include "no.xpm"
#include "help.xpm"
#include "print.xpm"
#include "next.xpm"
#include "previous.xpm"

/* Global structure and variables */
extern Xdialog_data Xdialog;

/* Fixed font and characters size (in pixels) initialisation */

static GdkFont *fixed_font;
static gint xmult= XSIZE_MULT;
static gint ymult= YSIZE_MULT;

static void font_init(void) {
	fixed_font = gdk_font_load(DEFAULT_FIXED_FONT);
	if (fixed_font != NULL) {
		xmult = gdk_char_width(fixed_font, 'A');
		ymult = gdk_char_height(fixed_font, 'A');
		ymult += ymult/2;
	}
}

/* Some useful functions to setup GTK menus... */

static void open_window(void)
{
	GtkWidget *window;

	if (strlen(Xdialog.rc_file) != 0)
		gtk_rc_parse(Xdialog.rc_file);

	font_init();

	/* Open a new GTK top-level window */
	window = Xdialog.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

	if (Xdialog.wmclass[0] != 0)
		gtk_window_set_wmclass(GTK_WINDOW(window), Xdialog.wmclass,
				       Xdialog.wmclass);

	/* Set default events handlers */
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
			   GTK_SIGNAL_FUNC(destroy_event), NULL);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
			   GTK_SIGNAL_FUNC(delete_event), NULL);

	/* Set the window title */
	gtk_window_set_title(GTK_WINDOW(window), Xdialog.title);

	/* Set the window size */
	if (Xdialog.xsize != 0 && Xdialog.ysize != 0) {
		if (Xdialog.size_in_pixels)
			gtk_window_set_default_size(GTK_WINDOW(window),
						    Xdialog.xsize,
						    Xdialog.ysize);
		else
			gtk_window_set_default_size(GTK_WINDOW(window),
						    Xdialog.xsize*xmult,
						    Xdialog.ysize*ymult);
	}

	/* Set the internal border so that the child widgets do not
	 * expand to the whole window (prettier) */
	gtk_container_set_border_width(GTK_CONTAINER(window), xmult/2);

	/* Allow the window to grow, shrink and auto-shrink */
	gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, TRUE);

	/* Set the window placement policy */
	switch (Xdialog.placement) {
		case WINDOW_CENTERED:
			gtk_window_set_position(GTK_WINDOW(window),
						GTK_WIN_POS_CENTER);
			break;
		case WINDOW_UNDER_MOUSE:
			gtk_window_set_position(GTK_WINDOW(window),
						GTK_WIN_POS_MOUSE);
			break;
		case WINDOW_AUTO_PLACEMENT:
			gtk_window_set_position(GTK_WINDOW(window),
						GTK_WIN_POS_NONE);
			break;
	}

	/* Create the root vbox widget in which all other boxes will
	 * be packed. By setting the "homogeneous" parameter to false,
	 * we allow packing with either gtk_box_pack_start() or
	 * gtk_box_pack_end() and disallow any automatic repartition
	 * of additional space between the child boxes (each box may
	 * therefore be set so to expand or not): this is VERY important
	 * and should not be changed !
	 */
	Xdialog.vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window), Xdialog.vbox);
	gtk_widget_show(Xdialog.vbox);

	gtk_widget_realize(Xdialog.window);

	Xdialog.exit_code = 255;
}

static void set_separator(void)
{
	GtkWidget *separator;

	separator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(Xdialog.vbox), separator, FALSE,
			   TRUE, ymult/3);
	gtk_widget_show(separator);
}

static void set_backtitle(gboolean sep_flag)
{
	GtkWidget *label;
	GtkWidget *hbox;

	if (strlen(Xdialog.backtitle) == 0)
		return;

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(Xdialog.vbox), hbox, FALSE, FALSE, ymult/3);
	label = gtk_label_new(Xdialog.backtitle);
	gtk_label_set_line_wrap(GTK_LABEL(label), Xdialog.wrap);
	gtk_container_add(GTK_CONTAINER(hbox), label);
	gtk_widget_show(hbox);
	gtk_widget_show(label);
	if (sep_flag)
		set_separator();
}

static GtkWidget *set_label(gchar *text, gboolean expand)
{
	GtkWidget *label;
	GtkWidget *hbox;
	GdkBitmap *mask;
	GdkColor  transparent;
	GdkPixmap *pixmap;
	GtkWidget *icon;

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(Xdialog.vbox), hbox, expand, TRUE,
			   ymult/3);

	if (Xdialog.icon) {
		pixmap = gdk_pixmap_create_from_xpm(Xdialog.window->window,
						    &mask, &transparent,
						    Xdialog.icon_file);
		if (pixmap != NULL) {
			icon = gtk_pixmap_new(pixmap, mask);
			gdk_pixmap_unref(pixmap);
			gdk_pixmap_unref(mask);
			gtk_box_pack_start(GTK_BOX(hbox), icon, FALSE, FALSE, 2);
/*			gtk_container_add(GTK_CONTAINER(hbox), icon); */
			gtk_widget_show(icon);
		}
	}

	label = gtk_label_new(text);
	gtk_label_set_line_wrap(GTK_LABEL(label), Xdialog.wrap);
	switch (Xdialog.justify) {
		case JUSTIFY_LEFT:
			gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
			break;
		case JUSTIFY_RIGHT:
			gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
			break;
		case JUSTIFY_CENTER:
			gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
			break;
	}
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 2);
/*	gtk_container_add(GTK_CONTAINER(hbox), label); */

	gtk_widget_show(hbox);
	gtk_widget_show(label);

	return label;
}

static GtkWidget *set_secondary_label(gchar *text, gboolean expand)
{
	GtkWidget *label;

	label = gtk_label_new(text);

	gtk_label_set_line_wrap(GTK_LABEL(label), Xdialog.wrap);
	switch (Xdialog.justify) {
		case JUSTIFY_LEFT:
			gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
			break;
		case JUSTIFY_RIGHT:
			gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
			break;
		case JUSTIFY_CENTER:
			gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
			break;
	}

	gtk_box_pack_start(GTK_BOX(Xdialog.vbox), label, expand, TRUE, ymult/3);
	gtk_widget_show(label);

	return label;
}

static GtkWidget *set_hbuttonbox(void)
{
	GtkWidget *hbuttonbox;

	hbuttonbox = gtk_hbutton_box_new();
	gtk_box_pack_end(GTK_BOX(Xdialog.vbox), hbuttonbox, FALSE, FALSE,
			 ymult/4);
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox),
				  GTK_BUTTONBOX_SPREAD);
	gtk_widget_show(hbuttonbox);

	return hbuttonbox;
}

static GtkWidget *set_button(gchar *text, gpointer buttonbox, gint event,
			     gboolean grab_default)
{
	GtkWidget *button;
	GdkBitmap *mask;
	GdkColor  transparent;
	GdkPixmap *pixmap;
	GtkWidget *icon;
	GtkWidget *hbox;
	GtkWidget *label;
	gchar	  **xpm = NULL;

	if (Xdialog.buttons_style != TEXT_ONLY) {
		if (!strcmp(text, OK) || !strcmp(text, YES))
			xpm = yes_xpm;
		else if (!strcmp(text, CANCEL) || !strcmp(text, NO))
			xpm = no_xpm;
		else if (!strcmp(text, HELP))
			xpm = help_xpm;
		else if (!strcmp(text, PRINT))
			xpm = print_xpm;
		else if (!strcmp(text, NEXT) || !strcmp(text, ADD))
			xpm = next_xpm;
		else if (!strcmp(text, PREVIOUS) || !strcmp(text, REMOVE))
			xpm = previous_xpm;
	}

	if (xpm == NULL)	/* buttons with text only */
		button = gtk_button_new_with_label(text);
	else {			/* buttons with icons */
		button = gtk_button_new();
		hbox = gtk_hbox_new(FALSE, 2);
		gtk_container_add(GTK_CONTAINER(button), hbox);
		gtk_widget_show(hbox);

		pixmap = gdk_pixmap_create_from_xpm_d(Xdialog.window->window,
						      &mask, &transparent,
						      xpm);
		icon = gtk_pixmap_new(pixmap, mask);
		gdk_pixmap_unref(pixmap);
		gdk_pixmap_unref(mask);
		/* gtk_box_pack_start(GTK_BOX(hbox), icon, FALSE, FALSE, 2); */
		gtk_container_add(GTK_CONTAINER(hbox), icon);
		gtk_widget_show(icon);

		if (Xdialog.buttons_style != ICON_ONLY) {	/* icons + text */
			label = gtk_label_new(text);
			/* gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0); */
			gtk_container_add(GTK_CONTAINER(hbox), label);
			gtk_widget_show(label);
		}
	}

	gtk_container_add(GTK_CONTAINER(buttonbox), button);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);

	switch (event) {
		case 0:
			gtk_signal_connect_after(GTK_OBJECT(button), "clicked",
						 GTK_SIGNAL_FUNC(exit_ok),
						 NULL);
			break;
		case 1:
			gtk_signal_connect_after(GTK_OBJECT(button), "clicked",
						 GTK_SIGNAL_FUNC(exit_cancel),
						 NULL);
			break;
		case 2:
			gtk_signal_connect_after(GTK_OBJECT(button), "clicked",
						 GTK_SIGNAL_FUNC(exit_help),
						 NULL);
			break;
		case 3:
			gtk_signal_connect_after(GTK_OBJECT(button), "clicked",
						 GTK_SIGNAL_FUNC(exit_previous),
						 NULL);
			break;
		case 4:
			gtk_signal_connect_after(GTK_OBJECT(button), "clicked",
						 GTK_SIGNAL_FUNC(print_text),
						 NULL);
			break;
	}
	if (grab_default)
		gtk_widget_grab_default(button);

	gtk_widget_show(button);

	return button;
}

static GtkWidget *set_all_buttons(gboolean print)
{
	GtkWidget *hbuttonbox; 
	GtkWidget *button_ok = NULL;

	hbuttonbox = set_hbuttonbox();

	if (Xdialog.wizard)
		set_button(PREVIOUS , hbuttonbox, 3, FALSE);
	else
		button_ok = set_button(OK, hbuttonbox, 0, !Xdialog.default_no);
	if (Xdialog.cancel_button)
		set_button(CANCEL , hbuttonbox, 1, Xdialog.default_no && !Xdialog.wizard);
	if (Xdialog.wizard)
		button_ok = set_button(NEXT, hbuttonbox, 0, TRUE);
	if (Xdialog.help)
		set_button(HELP, hbuttonbox, 2, FALSE);
	if (print && Xdialog.print)
		set_button(PRINT, hbuttonbox, 4, FALSE);

	return button_ok;
}

static GtkWidget *set_scrollable_text(gpointer vbox)
{
	GtkWidget *hbox;
	GtkWidget *text;
	GtkWidget *vscrollbar;
	GtkStyle *style;

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, xmult/2);
	gtk_widget_show(hbox);
  
	text = gtk_text_new(NULL, NULL);
	gtk_widget_set_usize(text, 40*xmult, 15*ymult);
	gtk_box_pack_start(GTK_BOX(hbox), text, TRUE, TRUE, 0);
	if (Xdialog.fixed_font) {
		style = gtk_style_new();
	  	gdk_font_unref(style->font);
		style->font = fixed_font;
	 	gtk_widget_push_style(style);     
	   	gtk_widget_set_style(GTK_WIDGET(text), style);
	   	gtk_widget_pop_style();
	}
	gtk_widget_show(text);

	/* Add a vertical scrollbar to the GtkText widget */
	vscrollbar = gtk_vscrollbar_new(GTK_TEXT(text)->vadj);
	gtk_box_pack_start(GTK_BOX(hbox), vscrollbar, FALSE, FALSE, 0);
	gtk_widget_show(vscrollbar);

	return text;
}

static GtkWidget *set_spin_button(GtkWidget *hbox, gint max, gint deflt,
				  gboolean with_separator)
{
	GtkAdjustment *adjust;
	GtkWidget *spin;
	GtkWidget *label;

	adjust = (GtkAdjustment *) gtk_adjustment_new(deflt, 0, max, 1, 1, 0);

	spin = gtk_spin_button_new(adjust, 0.5, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 0);
	gtk_widget_show(spin);
	if (with_separator) {
		label = gtk_label_new(":");
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
		gtk_widget_show(label);
	}

	return spin;
}

/*
 * The Xdialog widgets...
 */

void create_msgbox(gchar *optarg, gboolean yesno)
{
	GtkWidget *hbuttonbox; 

	open_window();

	set_backtitle(TRUE);
	set_label(optarg, TRUE);

	hbuttonbox = set_hbuttonbox();
	if (yesno) {
		if (Xdialog.wizard) {
			set_button(PREVIOUS , hbuttonbox, 3, FALSE);
			if (Xdialog.cancel_button)
				set_button(CANCEL , hbuttonbox, 1, FALSE);
			set_button(NEXT, hbuttonbox, 0, TRUE);
		} else {
			set_button(YES, hbuttonbox, 0, !Xdialog.default_no);
			set_button(NO , hbuttonbox, 1, Xdialog.default_no);
		}
	} else
		set_button(OK, hbuttonbox, 0, TRUE);
	if (Xdialog.help)
		set_button(HELP, hbuttonbox, 2, FALSE);
}


void create_infobox(gchar *optarg, gint timeout)
{
	GtkWidget *hbuttonbox;

	open_window();

	set_backtitle(TRUE);
	Xdialog.widget1 = set_label(optarg, TRUE);

	if (Xdialog.buttons) {
		hbuttonbox = set_hbuttonbox();
		set_button(timeout > 0 ? OK : CANCEL, hbuttonbox,
			   timeout > 0 ? 0 : 1, TRUE);
		if (Xdialog.help)
			set_button(HELP, hbuttonbox, 2, FALSE);
	}

	if (timeout > 0)
		Xdialog.timer = gtk_timeout_add(timeout,
						infobox_timeout_exit, NULL);
	else
		Xdialog.timer = gtk_timeout_add(1000,
						(GtkFunction) infobox_timeout,
						NULL);
}


void create_gauge(gchar *optarg, gint percent)
{
	GtkWidget *align;
	GtkWidget *pbar;
	GtkAdjustment *adj;
	int value;

	if (percent < 0)
		value = 0;
	else if (percent > 100)
		value = 100;
	else
		value = percent;

	open_window();

	set_backtitle(TRUE);
	Xdialog.widget2 = set_label(optarg, TRUE);

	align = gtk_alignment_new(0.5, 0.5, 0.8, 0);
	gtk_box_pack_start(GTK_BOX(Xdialog.vbox), align, FALSE, FALSE,
			   ymult/2);
	gtk_widget_show(align);

	/* Create an Adjusment object to hold the range of the progress bar */
	adj = (GtkAdjustment *) gtk_adjustment_new(0, 0, 100, 0, 0, 0);

	/* Set up the progress bar */
	pbar = Xdialog.widget1 = gtk_progress_bar_new_with_adjustment(adj);
	/* Set the start value and the range of the progress bar */
	gtk_progress_configure(GTK_PROGRESS(pbar), value, 0, 100);
	/* Set the format of the string that can be displayed in the
	 * trough of the progress bar:
	 * %p - percentage
	 * %v - value
	 * %l - lower range value
	 * %u - upper range value */
	gtk_progress_set_format_string(GTK_PROGRESS(pbar), "%p%%");
	gtk_progress_set_show_text(GTK_PROGRESS(pbar), TRUE);
	gtk_container_add(GTK_CONTAINER(align), pbar);
	gtk_widget_show(pbar);

	/* Add a timer callback to update the value of the progress bar */
	Xdialog.timer = gtk_timeout_add(1000, gauge_timeout, NULL);
}


void create_tailbox(gchar *optarg)
{
	GtkWidget *text;

	open_window();

	set_backtitle(FALSE);

	text = Xdialog.widget1 = set_scrollable_text(Xdialog.vbox);
	gtk_signal_connect(GTK_OBJECT(text), "key_press_event",
			   GTK_SIGNAL_FUNC(tailbox_keypress), NULL);

	if (strcmp(optarg, "-") == 0)
		Xdialog.file = stdin;
	else
		Xdialog.file = fopen(optarg, "r");

	if (Xdialog.file == NULL) {
		fprintf(stderr, "Xdialog: can't open %s\n", optarg);
		exit(255);
	}

	if (Xdialog.buttons)
		set_all_buttons(TRUE);

	Xdialog.timer = gtk_timeout_add(1000, (GtkFunction) tailbox_timeout,
					NULL);
}


void create_inputbox(gchar *optarg, gchar *options[], gint entries)
{
	GtkWidget *entry;
	GtkWidget *button_ok = NULL;
	gchar deftext[MAX_INPUT_DEFAULT_LENGTH];

	open_window();

	set_backtitle(TRUE);
	set_label(optarg, TRUE);

	if (entries > 1) {
		set_separator();
		set_secondary_label(options[0], FALSE);
		strcpysafe(deftext, options[1], MAX_INPUT_DEFAULT_LENGTH);
	} else {
		if (options[0] != NULL)
			strcpysafe(deftext, options[0], MAX_INPUT_DEFAULT_LENGTH);
		else
			deftext[0] = 0;
	}

	entry = Xdialog.widget1 = gtk_entry_new();
	gtk_signal_connect(GTK_OBJECT(entry), "key_press_event",
			   GTK_SIGNAL_FUNC(input_keypress), NULL);
	if (Xdialog.passwd >= entries)
		gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
	gtk_entry_set_text(GTK_ENTRY(entry), deftext);
	gtk_box_pack_start(GTK_BOX(Xdialog.vbox), entry, TRUE, TRUE, 0);
	gtk_widget_grab_focus(entry);
	gtk_widget_show(entry);

	Xdialog.widget2 = Xdialog.widget3 = NULL;

	if (entries > 1) {
		set_secondary_label(options[2], FALSE);

		entry = Xdialog.widget2 = gtk_entry_new();
		gtk_signal_connect(GTK_OBJECT(entry), "key_press_event",
				   GTK_SIGNAL_FUNC(input_keypress), NULL);
		if (Xdialog.passwd >= entries - 1)
			gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
		gtk_entry_set_text(GTK_ENTRY(entry), options[3]);
		gtk_box_pack_start(GTK_BOX(Xdialog.vbox), entry, TRUE, TRUE, 0);
		gtk_widget_show(entry);
	}

	if (entries > 2) {
		set_secondary_label(options[4], FALSE);

		entry = Xdialog.widget3 = gtk_entry_new();
		gtk_signal_connect(GTK_OBJECT(entry), "key_press_event",
				   GTK_SIGNAL_FUNC(input_keypress), NULL);
		if (Xdialog.passwd > 0)
			gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
		gtk_entry_set_text(GTK_ENTRY(entry), options[5]);
		gtk_box_pack_start(GTK_BOX(Xdialog.vbox), entry, TRUE, TRUE, 0);
		gtk_widget_show(entry);
	}

	if (Xdialog.buttons) {
		button_ok = set_all_buttons(FALSE);
		gtk_signal_connect(GTK_OBJECT(button_ok), "clicked",
				   GTK_SIGNAL_FUNC(inputbox_ok), NULL);
	}

	if (Xdialog.interval > 0)
		Xdialog.timer = gtk_timeout_add(Xdialog.interval, inputbox_timeout, NULL);
}


void create_textbox(gchar *optarg, gboolean editable)
{
	GtkWidget *text;
	GtkWidget *button_ok = NULL;
	FILE *infile;

	open_window();

	set_backtitle(FALSE);

	text = Xdialog.widget1 = set_scrollable_text(Xdialog.vbox);

	/* Fill the GtkText with the text */
	gtk_text_freeze(GTK_TEXT(text));
	if (strcmp(optarg, "-") == 0)
		infile = stdin;
	else
		infile = fopen(optarg, "r");
	if (infile) {
		char buffer[1024];
		int nchars;

		while (TRUE) {
			nchars = fread(buffer, 1, 1024, infile);
			gtk_text_insert(GTK_TEXT(text), NULL, NULL,
					NULL, buffer, nchars);
			if (nchars < 1024)
				break;
		}
		if (infile != stdin)
			fclose(infile);
	}
	gtk_text_thaw(GTK_TEXT(text));

	/* Set the editable flag depending on what we want (text or edit box) */
	gtk_text_set_editable(GTK_TEXT(text), editable);
	gtk_widget_grab_focus(text);

	/* Set the buttons */
	if (Xdialog.buttons || editable)
		button_ok = set_all_buttons(TRUE);

	if (editable) {
		gtk_signal_connect(GTK_OBJECT(button_ok), "clicked",
				   GTK_SIGNAL_FUNC(editbox_ok), NULL);
		/* gtk_text_set_word_wrap(GTK_TEXT(text), wrap); */
	}
}


void create_rangebox(gchar *optarg, gchar *options[], gint ranges)
{
	GtkWidget *align;
	GtkAdjustment *adj;
	GtkWidget *hscale;
	GtkWidget *button_ok;
	gint min, max, deflt;

	open_window();

	set_backtitle(TRUE);
	set_label(optarg, TRUE);

	if (ranges > 1) {
		set_separator();
		set_secondary_label(options[0], FALSE);
		min = atoi(options[1]);
		max = atoi(options[2]);
		deflt = atoi(options[3]);
	} else {
		min = atoi(options[0]);
		max = atoi(options[1]);
		if (options[2] != NULL)
			deflt = atoi(options[2]);
		else
			deflt = min;
	}

	align = gtk_alignment_new(0.5, 0.5, 0.8, 0);
	gtk_box_pack_start(GTK_BOX(Xdialog.vbox), align, FALSE, FALSE, 5);
	gtk_widget_show(align);

	/* Create an adjusment object to hold the range of the scale */
	Xdialog.widget1 = (GtkWidget *) gtk_adjustment_new(deflt, min, max, 1, 1, 0);
	adj = GTK_ADJUSTMENT((GtkObject *) Xdialog.widget1);
 	hscale = gtk_hscale_new(adj);
	gtk_scale_set_digits(GTK_SCALE(hscale), 0);
	gtk_container_add(GTK_CONTAINER(align), hscale);
	gtk_widget_show(hscale);

	Xdialog.widget2 = Xdialog.widget3 = NULL;

	if (ranges > 1) {
		set_secondary_label(options[4], FALSE);
		min = atoi(options[5]);
		max = atoi(options[6]);
		deflt = atoi(options[7]);

		align = gtk_alignment_new(0.5, 0.5, 0.8, 0);
		gtk_box_pack_start(GTK_BOX(Xdialog.vbox), align, FALSE, FALSE, 5);
		gtk_widget_show(align);

		/* Create an adjusment object to hold the range of the scale */
		Xdialog.widget2 = (GtkWidget *) gtk_adjustment_new(deflt, min, max, 1, 1, 0);
		adj = GTK_ADJUSTMENT((GtkObject *) Xdialog.widget2);
	 	hscale = gtk_hscale_new(adj);
		gtk_scale_set_digits(GTK_SCALE(hscale), 0);
		gtk_container_add(GTK_CONTAINER(align), hscale);
		gtk_widget_show(hscale);
	}

	if (ranges > 2) {
		set_secondary_label(options[8], FALSE);
		min = atoi(options[9]);
		max = atoi(options[10]);
		deflt = atoi(options[11]);

		align = gtk_alignment_new(0.5, 0.5, 0.8, 0);
		gtk_box_pack_start(GTK_BOX(Xdialog.vbox), align, FALSE, FALSE, 5);
		gtk_widget_show(align);

		/* Create an adjusment object to hold the range of the scale */
		Xdialog.widget3 = (GtkWidget *) gtk_adjustment_new(deflt, min, max, 1, 1, 0);
		adj = GTK_ADJUSTMENT((GtkObject *) Xdialog.widget3);
	 	hscale = gtk_hscale_new(adj);
		gtk_scale_set_digits(GTK_SCALE(hscale), 0);
		gtk_container_add(GTK_CONTAINER(align), hscale);
		gtk_widget_show(hscale);
	}

	button_ok = set_all_buttons(FALSE);
	gtk_signal_connect(GTK_OBJECT(button_ok), "clicked",
			   GTK_SIGNAL_FUNC(rangebox_exit), NULL);

	if (Xdialog.interval > 0)
		Xdialog.timer = gtk_timeout_add(Xdialog.interval, rangebox_timeout, NULL);
}


void create_itemlist(gchar *optarg, gint type, gchar *options[], gint list_size)
{
	GtkWidget *vbox2;
	GtkWidget *scrolled_window;
	GtkWidget *button_ok;
	GtkWidget *item;
	GtkRadioButton *radio = NULL;
	char temp[MAX_ITEM_LENGTH];
	int i;

	Xdialog_array(list_size);

	open_window();

	set_backtitle(TRUE);
	set_label(optarg, FALSE);

	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_container_set_border_width(GTK_CONTAINER(scrolled_window),
				       xmult/2);

	/* the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
	 * GTK_POLICY_AUTOMATIC will automatically decide whether you need
	 * scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
	 * there.  The first one is the horizontal scrollbar, the second, 
	 * the vertical. */
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	if (Xdialog.list_height > 0)
		gtk_widget_set_usize(scrolled_window, -2, 3*Xdialog.list_height*ymult);
	/* The dialog window is created with a vbox packed into it. */
	gtk_box_pack_start(GTK_BOX(Xdialog.vbox), scrolled_window, TRUE, TRUE, 0);
	gtk_widget_show(scrolled_window);

	vbox2 = gtk_vbox_new(FALSE, xmult);
	gtk_container_set_border_width(GTK_CONTAINER(vbox2), xmult);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), vbox2);
	gtk_widget_show(vbox2);

	button_ok = set_all_buttons(FALSE);
	gtk_signal_connect(GTK_OBJECT(button_ok), "clicked",
			   GTK_SIGNAL_FUNC(print_items), NULL);

	for (i = 0;  i < list_size; i++) {
		strcpysafe(Xdialog.array[i].tag, options[i*3], MAX_ITEM_LENGTH);
		if (Xdialog.tags) {
			strcpysafe(temp, options[i*3], MAX_ITEM_LENGTH);
			if (strlen(options[i*3+1]) != 0) {
				strcatsafe(temp, ": ", MAX_ITEM_LENGTH);
				strcatsafe(temp, options[i*3+1], MAX_ITEM_LENGTH);
			}
		} else
			strcpysafe(temp, options[i*3+1], MAX_ITEM_LENGTH);

		if (type == CHECKLIST)
			item = gtk_check_button_new_with_label(temp);
		else {
			item = gtk_radio_button_new_with_label_from_widget(radio, temp);
			radio = GTK_RADIO_BUTTON(item);
		}
		gtk_box_pack_start(GTK_BOX(vbox2), item, FALSE, FALSE, 0);
		gtk_widget_show(item);

		if (!strncasecmp(options[i*3+2], "on", 2))
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(item), TRUE);

		gtk_signal_connect(GTK_OBJECT(item), "toggled",
				   GTK_SIGNAL_FUNC(item_toggle), (gpointer)i);
		gtk_signal_connect(GTK_OBJECT(item), "button_press_event",
				   GTK_SIGNAL_FUNC(double_click_event), button_ok);
		gtk_signal_emit_by_name(GTK_OBJECT(item), "toggled");
	}

	if (Xdialog.interval > 0)
		Xdialog.timer = gtk_timeout_add(Xdialog.interval, itemlist_timeout, NULL);
}


void create_buildlist(gchar *optarg, gchar *options[], gint list_size)
{
	GtkWidget *hbox;
	GtkWidget *scrolled_window1;
	GtkWidget *scrolled_window2;
	GtkWidget *list1;
	GtkWidget *list2;
	GtkWidget *vbuttonbox;
	GtkWidget *button_add;
	GtkWidget *button_remove;
	GtkWidget *button_ok;
	GList *glist1 = NULL;
	GList *glist2 = NULL;
	GtkWidget *item;
	gint i;

	Xdialog_array(list_size);

	open_window();

	set_backtitle(TRUE);
	set_label(optarg, FALSE);

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_widget_show(hbox);
	gtk_box_pack_start(GTK_BOX(Xdialog.vbox), hbox, TRUE, TRUE, ymult/3);

	scrolled_window1 = gtk_scrolled_window_new(NULL, NULL);
	gtk_container_set_border_width(GTK_CONTAINER(scrolled_window1), 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window1),
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(hbox), scrolled_window1, TRUE, TRUE, 0);
	gtk_widget_set_usize(scrolled_window1, 20*xmult,
			     Xdialog.list_height > 0 ? 3*Xdialog.list_height*ymult/2 : -2);
	gtk_widget_show(scrolled_window1);
	Xdialog.widget1 = list1 = gtk_list_new();
	gtk_widget_show(list1);
	gtk_list_set_selection_mode(GTK_LIST(list1), GTK_SELECTION_MULTIPLE);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window1), list1);

	vbuttonbox = gtk_vbutton_box_new();
	gtk_widget_show(vbuttonbox);
	gtk_box_pack_start(GTK_BOX(hbox), vbuttonbox, FALSE, TRUE, 0);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(vbuttonbox), GTK_BUTTONBOX_SPREAD);
	button_add = set_button(ADD, vbuttonbox, -1, FALSE);
	gtk_signal_connect(GTK_OBJECT(button_add), "clicked",
			   GTK_SIGNAL_FUNC(add_to_list), NULL);
	button_remove = set_button(REMOVE, vbuttonbox, -1, FALSE);
	gtk_signal_connect(GTK_OBJECT(button_remove), "clicked",
			   GTK_SIGNAL_FUNC(remove_from_list), NULL);

	scrolled_window2 = gtk_scrolled_window_new(NULL, NULL);
	gtk_container_set_border_width(GTK_CONTAINER(scrolled_window2), 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window2),
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(hbox), scrolled_window2, TRUE, TRUE, 0);
	gtk_widget_set_usize(scrolled_window2, 20*xmult,
			     Xdialog.list_height > 0 ? 3*Xdialog.list_height*ymult/2 : -2);
	gtk_widget_show(scrolled_window2);
	Xdialog.widget2 = list2 = gtk_list_new();
	gtk_widget_show(list2);
	gtk_list_set_selection_mode(GTK_LIST(list2), GTK_SELECTION_MULTIPLE);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window2), list2);

	button_ok = set_all_buttons(FALSE);
	gtk_signal_connect(GTK_OBJECT(button_ok), "clicked",
			   GTK_SIGNAL_FUNC(print_list), NULL);

	for (i = 0;  i < list_size; i++) {
		strcpysafe(Xdialog.array[i].tag, options[i*3], MAX_ITEM_LENGTH);
		strcpysafe(Xdialog.array[i].name, options[i*3+1], MAX_ITEM_LENGTH);
		item = gtk_list_item_new_with_label(Xdialog.array[i].name);
		gtk_widget_show(item);
		Xdialog.array[i].widget = item;
		if (!strncasecmp(options[i*3+2], "on", 2))
			glist2 = g_list_append(glist2, item);
		else
			glist1 = g_list_append(glist1, item);
	}
	gtk_list_append_items(GTK_LIST(list1), glist1);
	gtk_list_append_items(GTK_LIST(list2), glist2);

	if (Xdialog.interval > 0)
		Xdialog.timer = gtk_timeout_add(Xdialog.interval, buildlist_timeout, NULL);
}


/* It looks like clists are particularly buggy in GTK+ (at least up to and
 * including GTK+ v1.2.8), so let's try to work around these (annoying !)
 * bugs.
 */
#define GTK_CLIST_BUGS_WORK_AROUND 1

void create_menubox(gchar *optarg, gchar *options[], gint list_size)
{
	GtkWidget *button_ok;
	GtkWidget *clist;
	GtkWidget *scrolled_window;
	static gchar *null_row[] = {NULL, NULL};
	gint rownum;
	int i;
#if GTK_CLIST_BUGS_WORK_AROUND
	int n = 0;
#endif

	Xdialog_array(list_size);
	Xdialog.array[0].state = 0;	/* The first row is selected as default */

	open_window();

	set_backtitle(TRUE);
	set_label(optarg, FALSE);

	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_container_set_border_width(GTK_CONTAINER(scrolled_window),
				       xmult/2);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	if (Xdialog.list_height > 0)
		gtk_widget_set_usize(scrolled_window, -2, 3*Xdialog.list_height*ymult/2);
	gtk_box_pack_start(GTK_BOX(Xdialog.vbox), scrolled_window, TRUE, TRUE, 0);
	gtk_widget_show(scrolled_window);

	clist = gtk_clist_new(2);
	gtk_clist_set_shadow_type(GTK_CLIST(clist), GTK_SHADOW_OUT);

	for (i = 0; i < list_size; i++) {
		strcpysafe(Xdialog.array[i].tag, options[i*2], MAX_ITEM_LENGTH);
		strcpysafe(Xdialog.array[i].name, options[i*2+1], MAX_ITEM_LENGTH);
#if GTK_CLIST_BUGS_WORK_AROUND
		if (strlen(Xdialog.array[i].tag) > n)
			n = strlen(Xdialog.array[i].tag);
#endif

		rownum = gtk_clist_append(GTK_CLIST(clist), null_row);
		gtk_clist_set_text(GTK_CLIST(clist), rownum, 0, Xdialog.array[i].tag);

	/* FIX ME !
	 * There is apparently a bug in GTK+ preventing to hide the first column
	 * of a CLIST... This is to say that in order to hide the tags when the
	 * --no-tags option is in force, we must put the <item>s text in both
	 * columns 0 and 1 and then hide column 1, instead of putting the <tag>s
	 * in column 0, the <item>s in column 1 and hiding the column 0...
	 * The callback function had also to be made independant of the text
	 * in the column 0 (which was supposed to hold the <tag>s name): it now
	 * retreives the tag name of the selected row right from the Xdialog.array
	 * structure instead of the CLIST itself (this is not a problem though)...
	 */
#if GTK_CLIST_BUGS_WORK_AROUND
		gtk_clist_set_text(GTK_CLIST(clist), rownum,
				   Xdialog.tags ? 1 : 0, Xdialog.array[i].name);
#else
		gtk_clist_set_text(GTK_CLIST(clist), rownum, 1, Xdialog.array[i].name);
#endif
	}

	/* FIX ME !
	 * There are indeed problems with clist & colunm 0 !
	 * Even gtk_clist_optimal_column_width does not work on column 0 :-(
	 */
#if GTK_CLIST_BUGS_WORK_AROUND
	gtk_clist_set_column_width(GTK_CLIST(clist), 0, (n+2)*xmult);
#else
	gtk_clist_optimal_column_width(GTK_CLIST(clist), 0);
#endif
	gtk_clist_optimal_column_width(GTK_CLIST(clist), 1);

	if (!Xdialog.tags)
#if GTK_CLIST_BUGS_WORK_AROUND
		gtk_clist_set_column_visibility(GTK_CLIST(clist), 1, FALSE);
#else
		gtk_clist_set_column_visibility(GTK_CLIST(clist), 0, FALSE);
#endif

	gtk_clist_select_row(GTK_CLIST(clist), 0, 0);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window),
					      clist);
	gtk_signal_connect(GTK_OBJECT(clist), "select_row",
			   GTK_SIGNAL_FUNC(item_select), NULL);
	gtk_widget_show(clist);

	button_ok = set_all_buttons(FALSE);
	gtk_signal_connect(GTK_OBJECT(button_ok), "clicked",
			   GTK_SIGNAL_FUNC(print_selection), NULL);
	gtk_signal_connect(GTK_OBJECT(clist), "button_press_event",
			   GTK_SIGNAL_FUNC(double_click_event), button_ok);

	if (Xdialog.interval > 0)
		Xdialog.timer = gtk_timeout_add(Xdialog.interval, menu_timeout, NULL);
}


void create_treeview(gchar *optarg, gchar *options[], gint list_size)
{
	GtkWidget *scrolled_window;
	GtkWidget *button_ok;
	GtkWidget *tree;
	GtkWidget *oldtree[MAX_TREE_DEPTH];
	GtkWidget *origtree;
	GtkWidget *subtree;
	GtkWidget *item;
	GtkWidget *selected = NULL;
	int depth = 0;
	int i;

	Xdialog_array(list_size);

	open_window();

	set_backtitle(TRUE);
	set_label(optarg, FALSE);

	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
  	gtk_container_set_border_width(GTK_CONTAINER(scrolled_window),
				       xmult/2);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	if (Xdialog.list_height > 0)
		gtk_widget_set_usize(scrolled_window, -2, 3*Xdialog.list_height*ymult/2);
	gtk_box_pack_start(GTK_BOX(Xdialog.vbox), scrolled_window, TRUE, TRUE, 0);
	gtk_widget_show(scrolled_window);

	tree = gtk_tree_new();
	gtk_widget_show(tree);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window),
					      tree);

	button_ok = set_all_buttons(FALSE);
	gtk_signal_connect(GTK_OBJECT(button_ok), "clicked",
			   GTK_SIGNAL_FUNC(print_selection), NULL);

	origtree = oldtree[0] = tree;

	for (i = 0 ; i < list_size ; i++) {
		strcpysafe(Xdialog.array[i].tag, options[4*i], MAX_ITEM_LENGTH);
		strcpysafe(Xdialog.array[i].name, options[4*i+1], MAX_ITEM_LENGTH);
		item = gtk_tree_item_new_with_label(Xdialog.array[i].name);
		Xdialog.array[i].widget = item;

		if (i > 0 && atoi(options[4*i+3]) < atoi(options[4*(i-1)+3])) {
			tree = oldtree[depth];
			depth--;
		}
		if (atoi(options[4*i+3]) == 0)
			tree = origtree;

		gtk_tree_append(GTK_TREE(tree), item);

		if (i+1 < list_size &&
		    atoi(options[4*i+3]) < atoi(options[4*(i+1)+3])) {
			if (atoi(options[4*i+3])+1 != atoi(options[4*(i+1)+3])) {
				fprintf(stderr,
					"Xdialog: You cannot increment the --treeview depth "\
					"by more than one each time !  Aborting...\n");
					exit(255);
			}
			subtree = gtk_tree_new();
/*			gtk_tree_set_view_mode(GTK_TREE(subtree),
					       GTK_TREE_VIEW_ITEM); */
			gtk_signal_connect(GTK_OBJECT(subtree), "button_press_event",
					   GTK_SIGNAL_FUNC(double_click_event),
					   button_ok);
			gtk_tree_item_set_subtree(GTK_TREE_ITEM(item), subtree);
			depth++;
			if (depth > MAX_TREE_DEPTH) {
				fprintf(stderr,
					"Xdialog: Max allowed depth for "\
					"--treeview is %d !  Aborting...\n",
					MAX_TREE_DEPTH);
					exit(255);
			}
			oldtree[depth] = tree;
			tree = subtree;
		}
		if (strncasecmp(options[4*i+2], "on", 2) == 0 && selected == NULL) {
			selected = item;
/*			gtk_tree_item_select(GTK_TREE_ITEM(selected)); */
			Xdialog.array[0].state = i;
		}
		gtk_widget_show(item);
	}

	gtk_tree_set_view_mode(GTK_TREE(origtree), GTK_TREE_VIEW_ITEM);
	gtk_tree_set_selection_mode(GTK_TREE(origtree), GTK_SELECTION_SINGLE);

	/* FIX ME !
	 * It looks like there is a bug in GTK+ (v1.2.6 to v1.2.8 at least...):
	 * the following code don't work properly: the selection, although
	 * triggering the selected item highlighting, is not taken into account
	 * (and may actually trigger a core dump when selecting other items)...
	 */
#define GTK_TREE_ITEM_SELECT_BUG 1
#if !GTK_TREE_ITEM_SELECT_BUG
	if (selected != NULL)
		gtk_tree_item_select(GTK_TREE_ITEM(selected));
#endif
	gtk_signal_connect(GTK_OBJECT(origtree), "selection_changed",
			   GTK_SIGNAL_FUNC(cb_selection_changed), origtree);
	gtk_signal_connect(GTK_OBJECT(origtree), "button_press_event",
			   GTK_SIGNAL_FUNC(double_click_event), button_ok);

	if (Xdialog.interval > 0)
		Xdialog.timer = gtk_timeout_add(Xdialog.interval, menu_timeout, NULL);
}


void create_filesel(gchar *optarg, gboolean dsel_flag)
{
	GtkWidget *filesel;

	font_init();

	filesel = Xdialog.window = gtk_file_selection_new(Xdialog.title);
	gtk_signal_connect(GTK_OBJECT(filesel), "destroy",
			   GTK_SIGNAL_FUNC(destroy_event), NULL);
	gtk_signal_connect(GTK_OBJECT(filesel), "delete_event",
			   GTK_SIGNAL_FUNC(delete_event), NULL);
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filesel)->cancel_button),
                                  "clicked", (GtkSignalFunc) exit_cancel, NULL);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel)->ok_button),
			   "clicked", (GtkSignalFunc) filesel_exit, filesel);
	Xdialog.exit_code = 255;

	/* Set the window size */
	if (Xdialog.xsize != 0 && Xdialog.ysize != 0) {
		if (Xdialog.size_in_pixels)
			gtk_window_set_default_size(GTK_WINDOW(filesel),
						    Xdialog.xsize,
						    Xdialog.ysize);
		else
			gtk_window_set_default_size(GTK_WINDOW(filesel),
						    Xdialog.xsize*xmult,
						    Xdialog.ysize*ymult);
	}

	/* Allow the window to grow, shrink and auto-shrink */
	gtk_window_set_policy(GTK_WINDOW(filesel), TRUE, TRUE, TRUE);

	/* Set placement policy */
	switch (Xdialog.placement) {
		case WINDOW_CENTERED:
			gtk_window_set_position(GTK_WINDOW(filesel),
						GTK_WIN_POS_CENTER);
			break;
		case WINDOW_UNDER_MOUSE:
			gtk_window_set_position(GTK_WINDOW(filesel),
						GTK_WIN_POS_MOUSE);
			break;
		case WINDOW_AUTO_PLACEMENT:
			gtk_window_set_position(GTK_WINDOW(filesel),
						GTK_WIN_POS_NONE);
			break;
	}

	gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), optarg);
	gtk_file_selection_complete(GTK_FILE_SELECTION(filesel), optarg);

	/* If we want a directory selector, then hide the file list parent */
	if (dsel_flag)
		gtk_widget_hide_all(GTK_WIDGET(GTK_WIDGET(GTK_FILE_SELECTION(filesel)->file_list)->parent));

	if (!Xdialog.buttons)
		gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(filesel));
	if (Xdialog.help)
		set_button(HELP, GTK_FILE_SELECTION(filesel)->action_area, 2, FALSE);
}


void create_calendar(gchar *optarg, gint day, gint month, gint year)
{
	GtkWidget *calendar;
	GtkWidget *button_ok;
	struct tm *localdate;
	time_t curr_time;
	gint flags;

	open_window();

	set_backtitle(TRUE);
	set_label(optarg, FALSE);

	flags = GTK_CALENDAR_SHOW_HEADING | GTK_CALENDAR_SHOW_DAY_NAMES;
	/* There is a bug in GTK+ v1.2.7 preventing the week numbers to show
	 * properly (all numbers are 0 !)...
	 */
	if (!(gtk_major_version == 1 && gtk_minor_version == 2 &&
	    gtk_micro_version == 7))
		flags = flags | GTK_CALENDAR_SHOW_WEEK_NUMBERS;

	calendar = Xdialog.widget1 = gtk_calendar_new();
	gtk_calendar_display_options(GTK_CALENDAR(calendar), flags);
	gtk_box_pack_start(GTK_BOX(Xdialog.vbox), calendar, TRUE, TRUE, 5);

	time(&curr_time);
	localdate = localtime(&curr_time);
	gtk_calendar_select_month(GTK_CALENDAR(calendar),
				  month == 0 ? localdate->tm_mon : month-1,
				  year == 0 ? localdate->tm_year+1900 : year);
	gtk_calendar_select_day(GTK_CALENDAR(calendar),
				day == 0 ? localdate->tm_mday : day);
	gtk_widget_show(calendar);


	button_ok = set_all_buttons(FALSE);
	gtk_signal_connect(GTK_OBJECT(button_ok), "clicked",
			   GTK_SIGNAL_FUNC(calendar_exit), NULL);
	gtk_signal_connect(GTK_OBJECT(calendar), "day_selected_double_click",
			   GTK_SIGNAL_FUNC(calendar_exit), NULL);
	gtk_signal_connect_after(GTK_OBJECT(calendar), "day_selected_double_click",
			   GTK_SIGNAL_FUNC(exit_ok), NULL);

	if (Xdialog.interval > 0)
		Xdialog.timer = gtk_timeout_add(Xdialog.interval, calendar_timeout, NULL);
}


void create_timebox(gchar *optarg)
{
	GtkWidget *frame;
	GtkWidget *hbox;
	GtkWidget *button_ok;
	struct tm *localdate;
	time_t curr_time;

	open_window();

	set_backtitle(TRUE);
	set_label(optarg, TRUE);

	frame = gtk_frame_new(TIME_FRAME_LABEL);
	gtk_frame_set_label_align(GTK_FRAME(frame), 0.5, 0.5);
	gtk_box_pack_start(GTK_BOX(Xdialog.vbox), frame, TRUE, TRUE, ymult/2);
	gtk_widget_show(frame);

	hbox = gtk_hbox_new(TRUE, 0);
	gtk_container_add(GTK_CONTAINER(frame), hbox);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), ymult);
	gtk_widget_show(hbox);

	time(&curr_time);
	localdate = localtime(&curr_time);

	Xdialog.widget1 = set_spin_button(hbox, 23, localdate->tm_hour, TRUE);
	Xdialog.widget2 = set_spin_button(hbox, 59, localdate->tm_min, TRUE);
	Xdialog.widget3 = set_spin_button(hbox, 59, localdate->tm_sec, FALSE);

	button_ok = set_all_buttons(FALSE);
	gtk_signal_connect(GTK_OBJECT(button_ok), "clicked",
			   GTK_SIGNAL_FUNC(timebox_exit), NULL);

	if (Xdialog.interval > 0)
		Xdialog.timer = gtk_timeout_add(Xdialog.interval, timebox_timeout, NULL);
}
